<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\Session\Session;


class AberturaCaixaController extends \Teknisa\MAN\Controllers\TeknisaCrud {
    
    protected $dataSourceName = 'vnd21000_aberturacaixa';
    protected $environment;
    protected $AberturaCaixaService;
    
    
    public function __construct($crudService, $environment, $AberturaCaixaService) {
        $this->crudService =$crudService;
        $this->environment = $environment;
        $this->AberturaCaixaService = $AberturaCaixaService;
    }
    
    public function findFilial(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'vnd21000_aberturacaixa_filial' ;
        $this->find($request,$response);
    }
    
    public function save(DTO\Request $request, DTO\Response $response){ $row = $request->getDataSet()->getRows();
        $isRegisterOpen = $this->AberturaCaixaService->isRegisterOpen($row[0]->get('CDFILIAL'),$row[0]->get('CDCAIXA'),$this->environment->getNrOrgTrab(),$row[0]->get('DTABERCAIX'));

        if($isRegisterOpen==true || $isRegisterOpen===null && ($row[0]->get('__is_new') == true)){
            $registerOpenInfo = $this->AberturaCaixaService->getAbertura(
                $row[0]->get('CDFILIAL'),
                $row[0]->get('CDCAIXA'),
                $this->environment->getNrOrgTrab(),
                $row[0]->get('DTABERCAIX') );
                
            $errorMessage = '';
            
            if(empty($registerOpenInfo['DTFECHCAIX'])) {
                $errorMessage = 'Este caixa se encontra aberto no dia: '. $registerOpenInfo['DTABERCAIX']; 
            } else {
                $errorMessage = 'Houve uma abertura de caixa neste horário. <br/>'.
                    'Consulte o histórico:<br/>'.
                    'Início: ' . 
                    $registerOpenInfo['DTABERCAIX'] . '<br/>' . 
                    'Fim: ' . 
                    $registerOpenInfo['DTFECHCAIX'];
            }
            
            $response->setError(new DTO\Response\Error($errorMessage, 1));
        } 
       else{
            $currentTime = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
            
            foreach($request->getDataSet()->getRows() as $key => &$row) {
                $isNew               = $row['__is_new']; 
                $row['IDATIVO']      = 'S';
                $row['NRORG']        = $this->environment->getNrOrgTrab();
                if($isNew){
                    $row['DTMOVTURCAIX'] = new \DateTime(null,new \DateTimeZone('America/Sao_Paulo'));
                    $row['DTMOVTURCAIX'] = $row['DTMOVTURCAIX']->format("d/m/Y");
                    $row['VRMOVIVEND']   = $this->moneyToDouble($row['VRMOVIVEND']);
                    $row['CDOPERABER']   = $row['CDOPERADOR'];
                    $row['DTFECHCAIX']   = null; 
                    $row['CDOPERFECH']   = null; 
                    $row['CDCAIXAORIG']  = null; 
                    $row['DTABERCAIORI'] = null; 
                }
                else{
                    $row['CDOPERABER'] = $row['CDOPERADOR'];
                }
            }
			$nomeCaixa = $row['NMCAIXA'];
            parent::save($request,$response);
            if($row['__is_new']){
                $NRSEQUMOV = $this->AberturaCaixaService->saveMovCaixa($row);
            }
			$return = array(
			    'DTABERCAIX' => $row['DTABERCAIX'],
			    'CDOPERABER' => $row['CDOPERABER']
			);
            $response->addDataSet(new DataSet($this->dataSourceName, $return));
            if($isNew)
                $response->addNotification(new Notification("Caixa {$nomeCaixa} aberto com sucesso!", Notification::TYPE_SUCCESS));
            else
                $response->addNotification(new Notification("Caixa {$nomeCaixa} editado com sucesso!", Notification::TYPE_SUCCESS));

        }
    }
    
    public function  moneyToDouble($value){
		if ($value){
			$value = preg_replace("/[^0-9,]/", "", $value);
			$value = str_replace ("," , "." , $value);			
		}
		else {
			$value = 0;
		}
		
		return floatval($value);
	}
    
    
}