<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;

use Teknisa\MAN\Services\CadastroIntegracaoService;
use Teknisa\MAN\Services\ParamKDSService;
use Teknisa\MAN\Services\AppsIntegrations\CallJenkins\IIntegracaoJenkinsJobs;
use Teknisa\MAN\Utilities;

class CadastroIntegracao extends \Teknisa\MAN\Controllers\TeknisaCrud {

    const dataSourceIntegracoes            = 'GetIntegracoesRepository';
    const dataSourceRecebimentos           = 'GetIntegracaoRepositoryRecebimentos';
    const dataSourceRecebimentosToken      = 'GetIntegracaoRepositoryRecebimentosToken';
    const dataSourceUnidades               = 'GetIntegracaoRepositoryUnidades';
    const dataSourceTokens                 = 'GetUnidadeTokens';
    const dataSourceTipoRecebimentosIntegracao  = 'RecebimentosDisponiveisIntegracaoRepo';
    const dataSourceTipoRecebimentosToken  = 'RecebimentosDisponiveisTokenRepo';
    
    protected $parameterBagColumnsDefault = array('NRORG', 'CDINTEGRACAO');
    protected $parameterBagUnidades = array('NRORG');
    protected $parameterBagTokens = array('NRORG', 'CDFILIAL', 'CDINTEGRACAO');
    protected $parameterBagRecebimentos = array('NRORG', 'CDINTEGRACAO','CDTPRECEINTE');
    protected $parameterBagTipoRecebimentos = array('NRORG');
    protected $parameterBagRecebimentosToken = array('NRORG', 'CDINTEGRACAO','CDTPRECEINTE', 'CDFILIAL', 'NMFILIAL');
    protected $parameterBagTipoRecebimentosToken = array('NRORG', 'CDINTEGRACAO');
    protected $parameterBagTipoRecebimentosIntegracao = array('NRORG', 'CDINTEGRACAO');
    
    private $jenkinsAPIService;
    
    public function __construct (
        $crudService, 
        $parameterBag, 
        $paramBag, 
        Manager $dataSourceManager,  
        Utilities\Util $util, 
        CadastroIntegracaoService $cadastroIntService,
        IIntegracaoJenkinsJobs $jenkinsAPIService) {
        $this->crudService              = $crudService;
        $this->parameterBag             = $parameterBag;
        $this->dataSourceManager        = $dataSourceManager;
        $this->util                     = $util;
        $this->cadService               = $cadastroIntService;
        $this->jenkinsAPIService        = $jenkinsAPIService; 
    }
    
    public function findRecebimentosDisponiveisIntegracao(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTipoRecebimentosIntegracao;
        $this->parameterBagColumns = $this->parameterBagTipoRecebimentosIntegracao;
        parent::find($request,$response);
    }
    
    public function findRecebimentosDisponiveisToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTipoRecebimentosToken;
        $this->parameterBagColumns = $this->parameterBagTipoRecebimentosToken;
        parent::find($request,$response);
    }
    
    public function saveIntegracao(DTO\Request $request, DTO\Response $response){
        $params = $request->getDataSet()->getRows();
        
        if($params[0]['__is_new'])
            $params[0]['CDINTEGRACAO'] = $this->util->newCode('INTEGRACAO', '3');
            
        $this->dataSourceName = self::dataSourceIntegracoes;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::save($request,$response);
    }
    
    public function saveRecebimentos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        $params = $request->getDataSet()->getRows();
        
        /* Deleta para depois salvar */
        if(!$params[0]['__is_new']) {
            $cdTipoReceChanged = $params[0]['CDTIPORECE'];
            $cdTpReceInte = $params[0]['CDTPRECEINTE'];
            
            $params[0]['CDTIPORECE'] = $params[0]['CDTIPORECE_BK'];
            $params[0]['CDTPRECEINTE'] = $params[0]['CDTPRECEINTE_BK'];
            $params[0]['NMTIPORECE'] = $params[0]['NMTIPORECE_BK'];
            
            $this->deleteDataSetWithoutNotification($request, $response);
            
            $params[0]['CDTIPORECE'] = $cdTipoReceChanged;
            $params[0]['CDTPRECEINTE'] = $cdTpReceInte;

            $params[0]['__is_new'] = true;
        }
        $this->parameterBagColumns = $this->parameterBagRecebimentos;
        parent::save($request,$response);
    }
    
    public function saveRecebimentosToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        $params = $request->getDataSet()->getRows();
        
        /* Deleta para depois salvar */
        if(!$params[0]['__is_new']) {
            $cdTipoReceChanged = $params[0]['CDTIPORECE'];
            $cdTpReceInte = $params[0]['CDTPRECEINTE'];
            
            $params[0]['CDTIPORECE'] = $params[0]['CDTIPORECE_BK'];
            $params[0]['CDTPRECEINTE'] = $params[0]['CDTPRECEINTE_BK'];
            $params[0]['NMTIPORECE'] = $params[0]['NMTIPORECE_BK'];
            
            $this->deleteDataSetWithoutNotification($request, $response);
            
            $params[0]['CDTIPORECE'] = $cdTipoReceChanged;
            $params[0]['CDTPRECEINTE'] = $cdTpReceInte;

            $params[0]['__is_new'] = true;
        }
        $this->parameterBagColumns = $this->parameterBagRecebimentosToken;
        parent::save($request,$response);
    }
    
    public function removeIntegracao(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracoes;
        self::deleteRowWithoutNotification($request,$response);
    }
    
    public function getIntegracoes(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceIntegracoes;
        $this->parameterBagColumns = $this->parameterBagColumnsDefault;
        parent::find($request,$response);
    }
    
    public function findRecebimentos(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        $this->parameterBagColumns = $this->parameterBagRecebimentos;
        parent::find($request,$response);
    }
    
    public function findRecebimentosToken(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        $this->parameterBagColumns = array('NRORG', 'CDINTEGRACAO', 'CDFILIAL');
        parent::find($request,$response);
    }
    
    public function removeRecebimentos(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentos;
        self::deleteDataSetWithoutNotification($request,$response);
    }
    
    public function removeRecebimentosToken(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceRecebimentosToken;
        self::deleteDataSetWithoutNotification($request,$response);
    }
    
    public function deleteDataSetWithoutNotification($request, $response) {
		try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function deleteRowWithoutNotification(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
        } catch (\Exception $e) {
            $response->setCriticalError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findUnidades(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceUnidades;
        $this->parameterBagColumns = $this->parameterBagUnidades;
        parent::find($request,$response);
    }
    
    public function findTokens(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = self::dataSourceTokens;
        $this->parameterBagColumns = $this->parameterBagTokens;
        parent::find($request,$response);
    }
    
    public function saveTokens(DTO\Request $request, DTO\Response $response){
        $params = $this->util->getParamsAsArray($request);
        
        if(!$params[0]['__is_new']) {
            try{
                $params = $this->util->getParamsAsArray($request);
                $retorno = $this->cadService->updateToken($params[0]);
            } catch (\Exception $e) {
                $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        }else {
            
            $this->dataSourceName = self::dataSourceTokens;
            $this->parameterBagColumns = $this->parameterBagTokens;
            parent::save($request,$response);
        }
    }
    
    public function removeTokens(DTO\Request\Row $request, DTO\Response $response){
        $params = $this->util->getParamsAsArray($request);
        
        try{
            $params = $this->util->getParamsAsArray($request);
            $retorno = $this->cadService->removeToken($params);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function startJenkinsJob(DTO\Request\Row $request, DTO\Response $response) {
        $params = $this->util->getParamsAsArray($request);

        $IDINTEGRACAO = $params['IDINTEGRACAO'];
            
        try {
            $result = $this->jenkinsAPIService->startJenkinsJob($IDINTEGRACAO);
            $response->addNotification(new Notification("O serviço foi iniciado com sucesso!", Notification::TYPE_SUCCESS)); 
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function stopJenkinsJob(DTO\Request\Row $request, DTO\Response $response) {
        $params = $this->util->getParamsAsArray($request);

        $IDINTEGRACAO = $params['IDINTEGRACAO'];

        try {
            $result = $this->jenkinsAPIService->stopJenkinsJob($IDINTEGRACAO);
            $response->addNotification(new Notification("O serviço foi iniciado com sucesso!", Notification::TYPE_SUCCESS)); 
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}