<?php
namespace Teknisa\MAN\Services\DataExporterImpl;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\Exception\InvalidFieldNameException;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\Query\ParameterTypeInferer;
use Zeedhi\DataExporter\Service\DataProvider;
use Zeedhi\Framework\DataSource\Configuration;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager\Exception;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\AssociatedWithDataSource;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;
use Zeedhi\Framework\DataSource\Operator\DefaultOperator;
use Zeedhi\Framework\DBAL\Driver\OCI8\OCI8Statement;
use Zeedhi\DataExporter\Service\DataProvider\SQLConnection as SQLConnection; 
/**
 * Manages the connection with SQL databases.
 */
class RMSQLConnection extends SQLConnection {
    
    protected $queryStr;
    
    protected function buildQueryConditions(FilterCriteria $filterCriteria, QueryBuilder $query) {
        $params = array();
        foreach($filterCriteria->getConditions() as $condition) {
            $params[$condition['columnName']] = $condition['value']; 
        }
        
        return $params;
    }

    protected function processWhereClause(FilterCriteria $filterCriteria, QueryBuilder $query) {
        return $this->buildQueryConditions($filterCriteria, $query);
    }
    
    protected function processFilterConditions(FilterCriteria $filterCriteria, QueryBuilder $query) {
        $params = $this->processWhereClause($filterCriteria, $query);
        $this->processPagination($filterCriteria, $query);
        return $params;
    }

    /**
     * Creates the query to retrieve data from the dataSource.
     *
     * @return QueryBuilder The query created.
     */
    protected function createSelectQuery() {
        return $this->createQueryBuilder()->select($this->queryStr);
    }

    /**
     * Builds the statement to be executed.
     *
     * @param FilterCriteria $filterCriteria The filter criteria that will be used.
     *
     * @throws
     */
    protected function buildStatement(FilterCriteria $filterCriteria) {
        try {
            ini_set('max_execution_time', 1800); 
            $query = $this->createSelectQuery();
            $params = $this->processFilterConditions($filterCriteria, $query);
            
            $queryStr = $this->dataSourceConfig->getQuery();
            
            foreach ($params as $key => $value) {
                $queryStr = str_replace(':'.$key, '\''.$value.'\'', $queryStr);
            }
            
            $this->queryStr =  $queryStr;
            
            $query = $this->createSelectQuery();
            $types = $this->inferTypes($params);
            
            $this->stmt = $query->execute();
        } catch (InvalidFieldNameException $e) {
            throw $this->rethrowException($e);
        }
    }

}